use common::CreateAchievement;
use common::DeleteAchievement;
use common::ToggleAchievement;
use reqwasm::http::Request;
use serde::Serialize;

pub struct RestService;

#[derive(thiserror::Error, Debug)]
pub enum RestServiceError {
    #[error("Failed to serialize/deserialize data: {0}")]
    SerdeError(#[from] serde_json::Error),

    #[error("Unexpected response")]
    UnexpectedResponse(reqwasm::http::Response),

    #[error("Failed request")]
    ReqwasmError(#[from] reqwasm::Error),
}

impl RestService {
    fn hostname() -> String {
        web_sys::window().unwrap().location().hostname().unwrap()
    }

    fn url_to(endpoint: &str) -> String {
        format!("http://{}:4000{endpoint}", Self::hostname())
    }

    async fn post_json<T: Serialize>(payload: T, path: &str) -> Result<(), RestServiceError> {
        let req = Request::post(&Self::url_to(path))
            .header("Content-Type", "application/json")
            .body(serde_json::to_string(&payload)?);

        let res = req.send().await;
        match res {
            Ok(response) => {
                let status_code = response.status();
                if (200..300).contains(&status_code) {
                    Ok(())
                } else {
                    log::warn!(
                        "Received unexpected status code: {}. Response: {:?}",
                        status_code,
                        response
                    );
                    Err(RestServiceError::UnexpectedResponse(response))
                }
            }
            Err(err) => {
                log::error!("Request failed: {:?}", err);
                Err(err.into())
            }
        }
    }

    pub async fn toggle_achievement(payload: ToggleAchievement) -> Result<(), RestServiceError> {
        Self::post_json(payload, "/toggle").await
    }

    pub async fn delete_achievement(payload: DeleteAchievement) -> Result<(), RestServiceError> {
        Self::post_json(payload, "/delete").await
    }

    pub async fn create_achievement(payload: CreateAchievement) -> Result<(), RestServiceError> {
        Self::post_json(payload, "/create").await
    }
}
